<html>
  <body>
    HTML formatted message to <strong><%= @recipient %></strong>.
  </body>
</html>
<html>
  <body>
    HTML formatted message to <strong><%= @recipient %></strong>.
  </body>
</html>
